from dios.itypes import *
from dios.options import *
import pandas as pd
import warnings


def _get_storage_class_values(cls):
    return [getattr(cls, c) for c in cls.__dict__ if not c.startswith("_")]


def throwMixedItypeErrWarn(msg):
    if dios_options[Options.mixed_itype_policy] in ['ignore', 'silent']:
        pass
    elif dios_options[Options.mixed_itype_policy] in ['error', 'err']:
        raise ItypeCastError(msg)
    else:
        warnings.warn(msg, ItypeWarning)
    return


# todo: make method an kwarg and remove dios_options access
def get_dios_to_dios_keys(keys, other):
    # we can assume that all keys are exist in self._data
    method = dios_options[Options.dios_to_dios_method]
    err_append = "consider changing dios.option['dios_to_dios_method']"

    # assign where possible, otherwise ignore
    if method == 0:
        keys = [k for k in keys if k in other.columns]

    # at least one key must be in self
    elif method == 1:
        keys = [k for k in keys if k in other.columns]
        if not keys:
            raise KeyError("src-DioS and dest-DioS need to share at least one key, " + err_append)

    # all keys must be in self, but more keys could exist in other,
    # eg. ``dios['a'] = dios[['a','b']]`` will update column-a but not column-b
    # eg. ``dios[['a','b']] = dios['a']`` will fail
    elif method == 2:
        fail = [k for k in keys if k not in other.columns]
        if fail:
            raise KeyError(f"{fail} are missing in the destiny-dios, " + err_append)

    # keys in both dios's must be equal
    elif method == 3:
        fail = set(keys).symmetric_difference(set(other.columns))
        if fail:
            raise KeyError(f"{fail} is not in both of src- and dest-dios, " + err_append)

    else:
        raise OptionsError(f"{method} is an invalid value for dios.option[dios_to_dios]")

    return keys
